<?php
namespace Zeedhi\Framework\Process;

class ProcessImpl{
    /* @const string PROCESS The process prefix for id generation. */
    private const PROCESS = 'process';
    /* @var string $id The process id. */
    private $id = null;
    /* @var callable $callable The process itself. */
    private $callable = null;

    /**
     * Constructor...
     *
     * @param callable $callback
     */
    public function __construct($callable, string $suffix = null){
        $this->callable = $callable;
        $this->id = uniqid(self::PROCESS).$suffix;
    }

    /**
     * {@inheritdoc}
     */
    public function getId(){
        return $this->id;
    }

    /**
     * {@inheritdoc}
     *
     * Closes session for preventing requisition block by
     * sessions writing deadlock, since PHP allows only
     * a single instance to write to session.
     *
     */
    public function run(){
        session_write_close();
        $done = true;
        try {
            $callback = $this->callable;
            $callback();
        } catch (\Exception $e){
            $done = false;
        }
        return $done;
    }
}